package com.gct.dlview.controller;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gct.common.core.result.Result;
import com.gct.dlview.entity.SampleDiagramResource;
import com.gct.dlview.service.GroupSampleRelationService;
import com.gct.dlview.service.SampleDiagramResourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author xsr
 * @since 2021-08-17
 */
@RestController
@RequestMapping("/groupSampleRelation")
public class GroupSampleRelationController {

    @Autowired
    GroupSampleRelationService groupSampleRelationService;

    @Autowired
    SampleDiagramResourceService sampleDiagramResourceService;

    @PreAuthorize("hasAuthority('build:write')")
    @PostMapping("/addSampleToGroup")
    public Result addSampleToGroup(@RequestBody JSONObject param) {
        JSONArray sampleList = param.getJSONArray("sampleList");
        String remark = param.getString("remark");
        String tag = param.getString("tag");
        Integer groupId = param.getInteger("groupId");
        return Result.ok(groupSampleRelationService.addSampleToGroup(sampleList, tag, remark, groupId));
    }

    @PreAuthorize("hasAuthority('build:write')")
    @GetMapping("/addSampleToGroupBatch")
    public Result addSampleToGroupBatch(
            @RequestParam String mark, @RequestParam String version,
            @RequestParam String batch, @RequestParam String gk,
            @RequestParam Integer impact, @RequestParam String tag,
            @RequestParam String remark, @RequestParam Integer groupId) {
        List<SampleDiagramResource> sampleList = sampleDiagramResourceService.getList(mark, version, batch, gk, impact);
        return Result.ok(groupSampleRelationService.addSampleToGroup((JSONArray) JSON.toJSON(sampleList), tag, remark, groupId));
    }

    @PreAuthorize("hasAuthority('build:query')")
    @GetMapping("/getGroupSample")
    public Result getGroupSample(@RequestParam Integer groupId, @RequestParam String remark, @RequestParam Integer curPage, @RequestParam Integer pageSize, @RequestParam Integer version) {
        return Result.ok(groupSampleRelationService.getGroupSample(groupId, remark, version, curPage, pageSize));
    }

    @PreAuthorize("hasAuthority('build:write')")
    @GetMapping("/updateGroupSampleTagSingle")
    public Result updateGroupSampleGkSingle(@RequestParam Integer groupId, @RequestParam Integer sampleId, @RequestParam String gk) {
        boolean res = groupSampleRelationService.updateGroupSampleTagSingle(groupId, sampleId, gk);
        return res ? Result.ok("更新成功") : Result.error("更新失败");
    }

    @PreAuthorize("hasAuthority('build:write')")
    @GetMapping("/updateGroupSampleTag")
    public Result updateGroupSampleGk(@RequestParam Integer groupId, @RequestParam String oldTag, @RequestParam String tag) {
        boolean res = groupSampleRelationService.updateGroupSampleTag(groupId, oldTag, tag);
        return res ? Result.ok("更新成功") : Result.error("更新失败");
    }

    @PreAuthorize("hasAuthority('build:write')")
    @GetMapping("/deleteGroupSampleTagSingle")
    public Result deleteGroupSampleSingle(@RequestParam Integer groupId, @RequestParam Integer sampleId) {
        boolean res = groupSampleRelationService.deleteGroupSampleTagSingle(groupId, sampleId);
        return res ? Result.ok("删除成功") : Result.error("删除失败");
    }

    @PreAuthorize("hasAuthority('build:write')")
    @GetMapping("/deleteGroupSampleTag")
    public Result deleteGroupSampleTag(@RequestParam Integer groupId, @RequestParam String tag) {
        boolean res = groupSampleRelationService.deleteGroupSampleTag(groupId, tag);
        return res ? Result.ok("删除成功") : Result.error("删除失败");
    }

    @PreAuthorize("hasAuthority('build:query')")
    @GetMapping("/getGroupCount")
    public Result getGroupCount(@RequestParam Integer groupId) {
        return Result.ok(groupSampleRelationService.getGroupCount(groupId));
    }

}

