package com.gct.dlview.controller;


import com.gct.common.core.result.Result;
import com.gct.dlview.entity.MachineResource;
import com.gct.dlview.entity.vo.PageBody;
import com.gct.dlview.service.MachineResourceService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lloyd
 * @since 2021-08-13
 */
@RestController
@RequestMapping("/machineResource")
public class MachineResourceController {

    @Autowired
    MachineResourceService service;

    @PreAuthorize("hasAuthority('train:query')")
    @ApiOperation(value = "获得机器配置")
    @PostMapping("/list")
    public Result getMachineListByCondition(@RequestBody MachineResource machineResource) {
        return service.getListByCondition(machineResource);
    }

    @PreAuthorize("hasAuthority('train:query')")
    @ApiOperation(value = "获得机器配置")
    @PostMapping("/oneById")
    public Result getMachineOneByCondition(@RequestBody MachineResource machineResource) {
        return service.getOneByCondition(machineResource);
    }

    @PreAuthorize("hasAuthority('train:query')")
    @ApiOperation(value = "获得机器配置 分页")
    @PostMapping("/page")
    public Result getMachinePageByCondition(@RequestBody PageBody<MachineResource> pageBody) {
        return service.getPageByCondition(pageBody);
    }

    @PreAuthorize("hasAuthority('train:query')")
    @PostMapping("/list/condition/machineName")
    @ApiOperation(value = "根据筛选内容获取制定的子集filters")
    public Result getFilterMachineNameByCondition(@RequestBody MachineResource machineResource) {
        return service.getFiguredFilterByCondition(machineResource, MachineResource.ColumnIdentify.machineName.getColumnName());
    }

    @PreAuthorize("hasAuthority('train:query')")
    @PostMapping("/list/condition/ip")
    @ApiOperation(value = "根据筛选内容获取制定的子集filters")
    public Result getFilterIPByCondition(@RequestBody MachineResource machineResource) {
        return service.getFiguredFilterByCondition(machineResource, MachineResource.ColumnIdentify.ipAddr.getColumnName());
    }

    @PreAuthorize("hasAuthority('train:query')")
    @PostMapping("/list/condition/mac")
    @ApiOperation(value = "根据筛选内容获取制定的子集filters")
    public Result getFilterMacByCondition(@RequestBody MachineResource machineResource) {
        return service.getFiguredFilterByCondition(machineResource, MachineResource.ColumnIdentify.macAddr.getColumnName());
    }

    @PreAuthorize("hasAuthority('train:query')")
    @PostMapping("/list/condition/location")
    @ApiOperation(value = "根据筛选内容获取制定的子集filters")
    public Result getFilterLocationByCondition(@RequestBody MachineResource machineResource) {
        return service.getFiguredFilterByCondition(machineResource, MachineResource.ColumnIdentify.locationRegion.getColumnName());
    }

    @PreAuthorize("hasAuthority('train:query')")
    @PostMapping("/list/condition/netType")
    @ApiOperation(value = "根据筛选内容获取制定的子集filters")
    public Result getFilterNetTypeByCondition(@RequestBody MachineResource machineResource) {
        return service.getFiguredFilterByCondition(machineResource, MachineResource.ColumnIdentify.netType.getColumnName());
    }

    @PreAuthorize("hasAuthority('train:write')")
    @PostMapping("/save")
    @ApiOperation(value = "保存结果 机器配置")
    public Result saveMachineResource(@RequestBody MachineResource machineResource) {
        return service.saveMachineResource(machineResource);
    }

    @PreAuthorize("hasAuthority('train:write')")
    @PostMapping("/deleteOne")
    @ApiOperation(value = "删除记录")
    public Result deleteMachineResource(@RequestBody MachineResource machineResource) {
        return service.deleteOne(machineResource);
    }

    @PreAuthorize("hasAuthority('train:write')")
    @PostMapping("/deleteBatch")
    @ApiOperation(value = "删除记录 批量")
    public Result deleteMachineResourceBatch(@RequestBody List<MachineResource> machineResource) {
        return service.deleteBatch(machineResource);
    }


}

