package com.gct.dlview.controller;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.core.result.Result;
import com.gct.dlview.entity.SampleDiagramResource;
import com.gct.dlview.service.SampleDiagramResourceService;
import com.netflix.client.http.HttpRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;

/**
 * <p>
 * 功图样本源 前端控制器
 * </p>
 *
 * @author lloyd
 * @since 2021-08-15
 */
@RestController
@RequestMapping("/sampleDiagram")
public class SampleDiagramResourceController {

    @Autowired
    SampleDiagramResourceService sampleDiagramResourceService;

    @PreAuthorize("hasAuthority('sample:query')")
    @GetMapping("/listByPage")
    public Result getListByPage(
            @RequestParam String mark, @RequestParam String version,
            @RequestParam String batch, @RequestParam String gk,
            @RequestParam Integer impact, @RequestParam Integer curPage,
            @RequestParam Integer pageSize) {
        JSONObject sample = sampleDiagramResourceService.getListByPage(mark, version, batch, gk, impact, curPage, pageSize);
        return Result.ok(sample);
    }

    @PreAuthorize("hasAuthority('sample:query')")
    @GetMapping("/getGkCount")
    public Result getGkCount(
            @RequestParam(required = false) String mark, @RequestParam(required = false) String version,
            @RequestParam(required = false) String batch, @RequestParam(required = false) String gk,
            @RequestParam(required = false) Integer impact) {
        return Result.ok(sampleDiagramResourceService.getGkCount(mark, version, batch, gk, impact));
    }

    @PreAuthorize("hasAuthority('sample:write')")
    @GetMapping("/updateSampleGk")
    public Result updateSampleGk(
            @RequestParam(required = false) String mark, @RequestParam(required = false) String version,
            @RequestParam(required = false) String batch, @RequestParam(required = false) Integer impact,
            @RequestParam String oldGk, @RequestParam String gk) {
        boolean res = sampleDiagramResourceService.updateSampleGk(mark, version, batch, impact, oldGk, gk);
        return res?Result.ok("更新成功"):Result.error("更新失败");
    }

    @PreAuthorize("hasAuthority('sample:write')")
    @GetMapping("/updateSampleGkSingle")
    public Result updateSampleGkSingle(
            @RequestParam Integer id,
            @RequestParam String oldGk, @RequestParam String gk) {
        boolean res = sampleDiagramResourceService.updateSampleGkSingle(id, oldGk, gk);
        return res?Result.ok("更新成功"):Result.error("更新失败");
    }

    @PreAuthorize("hasAuthority('sample:write')")
    @GetMapping("/deleteSample")
    public Result deleteSample(
            @RequestParam(required = false) String mark, @RequestParam(required = false) String version,
            @RequestParam(required = false) String batch, @RequestParam(required = false) Integer impact,
           @RequestParam String gk
    ){
        boolean res = sampleDiagramResourceService.deleteSample(mark, version, batch, impact, gk);
        return res?Result.ok("删除成功"):Result.error("删除失败");
    }

    @PreAuthorize("hasAuthority('sample:write')")
    @PostMapping("/sampleFileUpload")
    public Result sampleFileUpload(@RequestParam MultipartFile file,@RequestParam String mark,
                                   @RequestParam String version,@RequestParam String batch){
        try {
            sampleDiagramResourceService.dealSampleFile(file,mark,version,batch);
        } catch (IOException e) {
            return Result.error(file.getOriginalFilename()+" 处理出错 " + e.getMessage());
        }
        return Result.ok("上传成功");
    }

    @PreAuthorize("hasAuthority('sample:write')")
    @GetMapping("/updateSampleImpact")
    public Result updateSampleImpact(@RequestParam Integer id,@RequestParam Integer sampleImpact){
        boolean res = sampleDiagramResourceService.updateSampleImpact(id,sampleImpact);
        return res?Result.ok("修改成功"):Result.error("修改失败");
    }

}

