package com.gct.dlview.entity;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author xusirui 2021/9/8
 */
class StatCategory {
    public final static String stableStat = "稳定状态";
    public final static String controlStat = "控制发送状态";
    public final static String unstableStat = "非稳定状态";
}

public enum TaskStateEnum {

    INIT(0, "初始状态", StatCategory.unstableStat, null),
    RUNNING(1, "运行中", StatCategory.unstableStat, null),
    PAUSING(2, "暂停中", StatCategory.controlStat, "暂停"),//  1->2->3
    PAUSED(3, "已暂停", StatCategory.stableStat, null),
    CONTINUE(4, "恢复中", StatCategory.controlStat, "恢复"), // 3->4->1
    CLOSING(5, "关闭中", StatCategory.controlStat, "关闭"),// 1,3->5->6
    CLOSED(6, "已关闭", StatCategory.stableStat, null),
    RESTARTING(7, "重启中", StatCategory.controlStat, "重启"), //1,3,6,8,9->7->1
    FINISHED(8, "已完成", StatCategory.stableStat, null),
    INTERRUPT(9, "异常中断", StatCategory.stableStat, null),
    AUTO_RETRY(10, "自动重试", StatCategory.unstableStat, null);

    private final Integer id;
    private final String dec;
    private final String category;
    private final String operateName;

    private final static Map<TaskStateEnum, List<TaskStateEnum>> ruleMap;

    static {
        ruleMap = new HashMap<>();
        ruleMap.putIfAbsent(RUNNING, new ArrayList<>());
        ruleMap.get(RUNNING).add(PAUSING);
        ruleMap.get(RUNNING).add(CLOSING);
        ruleMap.get(RUNNING).add(RESTARTING);
        ruleMap.putIfAbsent(AUTO_RETRY, new ArrayList<>());
        ruleMap.get(AUTO_RETRY).add(PAUSING);
        ruleMap.get(AUTO_RETRY).add(CLOSING);
        ruleMap.get(AUTO_RETRY).add(RESTARTING);
        ruleMap.putIfAbsent(PAUSED, new ArrayList<>());
        ruleMap.get(PAUSED).add(CONTINUE);
        ruleMap.get(PAUSED).add(CLOSING);
        ruleMap.get(PAUSED).add(RESTARTING);
        ruleMap.putIfAbsent(CLOSED, new ArrayList<>());
        ruleMap.get(CLOSED).add(RESTARTING);
        ruleMap.putIfAbsent(FINISHED, new ArrayList<>());
        ruleMap.get(FINISHED).add(RESTARTING);
        ruleMap.putIfAbsent(INTERRUPT, new ArrayList<>());
        ruleMap.get(INTERRUPT).add(RESTARTING);
    }

    TaskStateEnum(Integer id, String dec, String category, String operateName) {
        this.id = id;
        this.dec = dec;
        this.category = category;
        this.operateName = operateName;
    }

    public static TaskStateEnum getById(Integer id) {
        for (int i = 0; i < values().length; i++) {
            if (id.equals(values()[i].getId())) return values()[i];
        }
        return INIT;
    }

    public static boolean isStable(Integer id) {
        return StatCategory.stableStat.equals(getById(id).getCategory());
    }

    public static TaskStateEnum getByDec(String dec) {
        for (int i = 0; i < values().length; i++) {
            if (dec.equals(values()[i].getDec())) return values()[i];
        }
        return INIT;
    }

    public Integer getId() {
        return id;
    }

    public String getDec() {
        return dec;
    }

    public String getCategory() {
        return category;
    }

    public String getOperateName() {
        return operateName;
    }

    public static JSONArray getReachableStateList(TaskStateEnum taskState) {
        JSONArray res = new JSONArray();
        ruleMap.get(taskState).forEach(x -> {
            JSONObject jo = new JSONObject();
            jo.put("id", x.getId());
            jo.put("operateName", x.getOperateName());
            res.add(jo);
        });
        return res;
    }
}