package com.gct.dlview.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.core.result.Result;
import com.gct.dlview.entity.DeepLearnTrainServer;
import com.gct.dlview.entity.MachineResource;
import com.gct.dlview.entity.vo.PageBody;
import com.gct.dlview.mapper.DeepLearnTrainServerMapper;
import com.gct.dlview.service.DeepLearnTrainServerService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.dlview.service.MachineResourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lloyd
 * @since 2021-08-15
 */
@Service
public class DeepLearnTrainServerServiceImpl extends ServiceImpl<DeepLearnTrainServerMapper, DeepLearnTrainServer> implements DeepLearnTrainServerService {

    @Autowired
    MachineResourceService machineResourceService;

    @Override
    public Result getPageByCondition(PageBody<DeepLearnTrainServer> pageBody) {
        if (Objects.isNull(pageBody) || Objects.isNull(pageBody.getPage()) || Objects.isNull(pageBody.getCondition()))
            return Result.error("输入参数错误");
        Page<DeepLearnTrainServer> data = baseMapper.getTrainServerByCondition(pageBody.getPage(), pageBody.getCondition());
        if (Objects.isNull(data) || data.getRecords().isEmpty()) return Result.error("无数据");
        else return Result.ok(data);
    }

    @Override
    public Result getOneByCondition(DeepLearnTrainServer condition) {
        if (Objects.isNull(condition))
            return Result.error("输入参数错误");
        List<DeepLearnTrainServer> data = baseMapper.getTrainServerByCondition(condition);
        if (Objects.isNull(data) || data.isEmpty()) return Result.error("无数据");
        return Result.ok(data.get(0));
    }

    @Override
    public Result getOneById(Serializable id) {
        if (Objects.isNull(id)) return Result.error("输入参数错误");
        DeepLearnTrainServer condition = new DeepLearnTrainServer();
        condition.setId((Integer) id);
        return getOneByCondition(condition);
    }

    @Override
    public Result deleteOneById(DeepLearnTrainServer condition) {
        if (Objects.isNull(condition) || Objects.isNull(condition.getId()))
            return Result.error("输入参数错误");
        int i = baseMapper.deleteById(condition.getId());
        if (i <= 0) return Result.error("删除失败");
        return Result.ok("删除成功");
    }

    @Override
    public Result deleteBatchById(List<DeepLearnTrainServer> condition) {
        if (Objects.isNull(condition) || condition.isEmpty())
            return Result.error("输入参数错误");
        List<Serializable> ids = new ArrayList<>();
        condition.forEach(item -> {
            if (Objects.nonNull(item) && item.getId() != null)
                ids.add(item.getId());
        });
        int i = baseMapper.deleteBatchIds(ids);
        if (i <= 0) return Result.error("删除失败");
        return Result.ok("删除成功");
    }

    @Override
    public Result saveOne(DeepLearnTrainServer deepLearnTrainServer) {
        if (Objects.isNull(deepLearnTrainServer))
            return Result.error("没有输入数据");
        if (Objects.isNull(deepLearnTrainServer.getId()) || deepLearnTrainServer.getId() <= 0)
            deepLearnTrainServer.setId(null);
        if (Objects.isNull(deepLearnTrainServer.getMachineId()) || deepLearnTrainServer.getMachineId() <= 0)
            return Result.error("没有选择正确设备");
        if (Objects.isNull(deepLearnTrainServer.getServerName()) || deepLearnTrainServer.getServerName().isEmpty())
            return Result.error("输入正确的服务名称");
        if (Objects.isNull(deepLearnTrainServer.getCreateTime()))
            deepLearnTrainServer.setCreateTime(LocalDateTime.now());
        deepLearnTrainServer.setUpdateTime(LocalDateTime.now());
        try {
            System.out.println(deepLearnTrainServer);
            if(Objects.isNull(deepLearnTrainServer.getId())){
                boolean save = this.save(deepLearnTrainServer);
                if (save) return Result.ok("保存成功");
                else return Result.error("保存失败");
            }else {
                if(this.removeById(deepLearnTrainServer.getId())) {
                    return this.save(deepLearnTrainServer)?Result.ok("更新成功"):Result.error("更新失败");
                }
                return Result.error("更新失败");
            }
        } catch (Exception e) {
            return Result.error("save failed:" + e.getMessage());
        }
    }

    @Override
    public boolean updateRunStateById(int state, Serializable id) {
        try {
            DeepLearnTrainServer server = baseMapper.selectById(id);
            if (Objects.isNull(server)) {
                log.warn("Not exited update events for id:" + id.toString());
                return false;
            }
            if (server.getRunState() == state) {
                log.debug("Not necessarily update same state id:" + id.toString() + ",state:" + state);
                return true;
            }
            server.setRunState(state);
            if (baseMapper.updateById(server) > 0) return true;
            else return false;
        } catch (Exception e) {
            log.error("Update state failed id:" + id.toString() + " , cause by " + e.getMessage());
            return false;
        }
    }

    @Override
    public HashMap<Serializable, DeepLearnTrainServer> getMapByIds(List<Serializable> id) {
        List<DeepLearnTrainServer> deepLearnTrainServers = baseMapper.selectBatchIds(id);
        if (Objects.isNull(deepLearnTrainServers) || deepLearnTrainServers.isEmpty()) return null;
        return deepLearnTrainServers.stream().collect(Collectors.toMap(DeepLearnTrainServer::getId, x -> x, (a, b) -> b, HashMap::new));
    }

    @Override
    public List<DeepLearnTrainServer> getAllList() {
        Map<Integer, MachineResource> map = new HashMap<>();
        machineResourceService.list().stream().forEach(x->map.put(x.getId(),x));
        List<DeepLearnTrainServer> list = baseMapper.selectList(new LambdaQueryWrapper<DeepLearnTrainServer>().isNotNull(DeepLearnTrainServer::getId));
        list.forEach(x->x.setIpAddr(map.get(x.getMachineId()).getIpAddr()).setLocalOriginPath(map.get(x.getMachineId()).getLocalOriginPath()));
        return list;
    }
}
