package com.gct.dlview.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.util.DateTimeUtil;
import com.gct.dlview.entity.GroupInfo;
import com.gct.dlview.entity.GroupSampleRelation;
import com.gct.dlview.entity.GroupSampleRelationIndex;
import com.gct.dlview.entity.TrainTask;
import com.gct.dlview.mapper.GroupInfoMapper;
import com.gct.dlview.service.GroupInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.dlview.service.GroupSampleRelationIndexService;
import com.gct.dlview.service.GroupSampleRelationService;
import com.gct.dlview.service.TrainTaskService;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 分组信息 服务实现类
 * </p>
 *
 * @author xsr
 * @since 2021-08-17
 */
@Service
public class GroupInfoServiceImpl extends ServiceImpl<GroupInfoMapper, GroupInfo> implements GroupInfoService {

    @Autowired
    GroupSampleRelationService groupSampleRelationService;

    @Autowired
    GroupSampleRelationIndexService groupSampleRelationIndexService;

    @Autowired
    TrainTaskService trainTaskService;

    //接口复用
    @Override
    public JSONArray getGroupInfoTree() {
        //List<Map<String,Object>> list = this.listMaps(new QueryWrapper<GroupInfo>().select("id \"value\",group_name \"label\",parent_group_id \"parentId\""));
        List<Map<String,Object>> list = baseMapper.getGroupInfoTree();
        JSONArray root = new JSONArray();
        Map<Integer,JSONObject> map = new HashMap<>();
        list.forEach(m->{
           JSONObject obj =  (JSONObject) JSON.toJSON(m);
           obj.put("updateTime", DateTimeUtil.formatString(LocalDateTime.parse(obj.get("updateTime").toString().substring(0,19),DateTimeUtil.GCT_DATE_TIME)));
            map.put(obj.getInteger("value"),obj);
            if (obj.getInteger("parentId")==0)root.add(obj);
        });
        list.forEach(m->{
            JSONObject parent = map.get(Integer.parseInt(m.get("parentId").toString()));
            if (parent!=null){
                JSONArray children = parent.getJSONArray("children");
                if (children==null){
                    children = new JSONArray();
                    parent.put("children",children);
                }
                children.add(map.get(Integer.parseInt(m.get("value").toString())));
            }
        });
        return root;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String addSubGroup(Integer parentGroupId, String remark, String groupName) {
        QueryWrapper<GroupInfo> wrapper = new QueryWrapper<GroupInfo>()
                .eq("group_name", groupName).eq("parent_group_id", parentGroupId);
        List<GroupInfo> groupList = baseMapper.selectList(wrapper);//获取刚加入的分组ID，并顺便判断名称是否重复
        if (groupList.size()!=0){
            return "子结点名称已存在";
        }
        GroupInfo groupInfo = new GroupInfo().setGroupName(groupName)
                .setParentGroupId(parentGroupId)
                .setRemark(remark)
                .setCreateTime(LocalDateTime.now())
                .setCreateUser(SecurityContextHolder.getContext().getAuthentication().getName());
        groupInfo.insert();
        Integer id = baseMapper.selectOne(wrapper).getId();//获取数据库自动生成的主键id groupId
        new GroupSampleRelationIndex().setGroupId(id).setVersionSerialCode(1).setVersionTag("新建结点，版本自动生成").insert();
        return "添加成功";
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public String deleteGroup(Integer groupId) {
        //判断是否为叶子
        List<GroupInfo> childs = baseMapper.selectList(new QueryWrapper<GroupInfo>().eq("parent_group_id", groupId));
        if (childs.size()!=0){
            Integer version = Integer.parseInt(
                    groupSampleRelationIndexService.getMap(
                    new QueryWrapper<GroupSampleRelationIndex>()
                            .select("max(version_serial_code)")
                            .eq("group_id",groupId)
                            .groupBy("group_id"))
                    .get("max").toString());
            QueryWrapper<GroupSampleRelation> wrapper = new QueryWrapper<GroupSampleRelation>()
                    .eq("group_id", groupId)
                    .eq("version_serial_code", version);
            Integer count = groupSampleRelationService.getBaseMapper().selectCount(wrapper);
            boolean isRemove = false;
            if(count>0)isRemove = groupSampleRelationService.remove(wrapper);
            return "删除"+(isRemove?"成功":"失败")+"当前最大版本"+version+"的样本数量为"+count;
        }
        //判断是否训练过（任务判断）
//        List<GroupSampleRelationIndex> versionList = groupSampleRelationIndexService.list(new QueryWrapper<GroupSampleRelationIndex>().eq("group_id", groupId));
//        if (versionList.size()>1)return "已训练的结点不支持删除操作";
        TrainTask one = trainTaskService.getOne(new QueryWrapper<TrainTask>().eq("train_sample_source_id", groupId));
        if (one!=null)return "已训练的结点不支持删除操作";
        // 删除结点及关联
        groupSampleRelationIndexService.remove(new QueryWrapper<GroupSampleRelationIndex>().eq("group_id", groupId));
        groupSampleRelationService.remove(new QueryWrapper<GroupSampleRelation>().eq("group_id",groupId));
        baseMapper.delete(new QueryWrapper<GroupInfo>().eq("id",groupId));
        return "删除成功";
    }
}
