package com.gct.dlview.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.enums.SqlMethod;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.core.toolkit.ReflectionKit;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.util.SGTUtil;
import com.gct.common.util.StringUtil;
import com.gct.dlview.entity.GroupSampleRelation;
import com.gct.dlview.entity.GroupSampleRelationIndex;
import com.gct.dlview.entity.SampleDiagramInterpretationTemp;
import com.gct.dlview.mapper.GroupSampleRelationMapper;
import com.gct.dlview.service.GroupSampleRelationIndexService;
import com.gct.dlview.service.GroupSampleRelationService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.dlview.service.SampleDiagramInterpretationTempService;
import com.gct.dlview.service.SampleDiagramResourceService;
import org.apache.ibatis.binding.MapperMethod;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author xsr
 * @since 2021-08-17
 */
@Service
public class GroupSampleRelationServiceImpl extends ServiceImpl<GroupSampleRelationMapper, GroupSampleRelation> implements GroupSampleRelationService {

    @Autowired
    SampleDiagramInterpretationTempService sampleDiagramInterpretationTempService;

    @Autowired
    SampleDiagramResourceService sampleDiagramResourceService;

    @Autowired
    GroupSampleRelationIndexService groupSampleRelationIndexService;

    private Map<String, Object> getVersionMsg(Integer groupId) {
        HashMap<String, Object> map = new HashMap<>();

        map.put("max", groupSampleRelationIndexService.getMap(new QueryWrapper<GroupSampleRelationIndex>()
                .select("max(version_serial_code)").eq("group_id", groupId)).get("max"));
        String versionTag = groupSampleRelationIndexService.getOne(new QueryWrapper<GroupSampleRelationIndex>()
                .eq("version_serial_code", map.get("max"))
                .eq("group_id", groupId)).getVersionTag();
        map.put("version_tag", versionTag);
        return map;
    }

    @Override
    public String addSampleToGroup(JSONArray sampleList, String tag, String remark, Integer groupId) {
        Map<String, Object> versionMsg = getVersionMsg(groupId);
        Integer version = Integer.parseInt(versionMsg.get("max").toString());
        Object versionTagStr = versionMsg.get("version_tag");
        String versionTag = versionTagStr == null ? null : versionTagStr.toString();
        String username = SecurityContextHolder.getContext().getAuthentication().getName();
        List<GroupSampleRelation> list = new ArrayList<>();
        for (int i = 0; i < sampleList.size(); i++) {
            JSONObject jo = sampleList.getJSONObject(i);
            String gk = jo.getString("gk");
            if (StringUtil.isBlankOrEmpty(gk))gk=jo.getString("graphicInterpretation");
            Integer id = jo.getInteger("id");
            GroupSampleRelation gsr = new GroupSampleRelation();
            // gsr.setId(0);
            gsr.setGroupId(groupId);
            gsr.setSampleId(id);
            gsr.setCreateTime(LocalDateTime.now());
            gsr.setCreateUser(username);
            gsr.setSampleType(1);
            gsr.setSampleTag1(tag != null && tag.length() > 0 ? tag : gk);
            gsr.setRemark(remark);
            gsr.setVersionSerialCode(version);
            gsr.setVersionTag(versionTag);
            list.add(gsr);
        }
        boolean flag = false;

        this.executeBatch(list,(sqlSession,entity)->{
            sqlSession.insert("insertOne",entity);
        });
        if (tag != null && !"".equals(tag)) {
            sampleDiagramInterpretationTempService.saveOrUpdate(
                    new SampleDiagramInterpretationTemp().setInterpretation(tag),
                    new QueryWrapper<SampleDiagramInterpretationTemp>()
                            .eq("interpretation", tag));
        }
        return flag ? "存在部分数据重复添加" : "添加成功";
    }

    @Override
    public Page<Map<String, Object>> getGroupSample(Integer groupId, String remark, Integer version, Integer curPage, Integer pageSize) {
//
        Page<Map<String, Object>> page = baseMapper.getGroupSample(new Page<>(curPage, pageSize), groupId, remark, version);
        page.getRecords().forEach(m -> {
            List<List<Double>> gt = SGTUtil.encodeToUseList(m.get("diagramContent").toString());
            m.put("diagramContent", gt);
            m.put("minS", gt.stream().min(Comparator.comparingDouble(p -> p.get(0))).get().get(0));
            m.put("maxS", gt.stream().max(Comparator.comparingDouble(p -> p.get(0))).get().get(0));
            m.put("minL", gt.stream().min(Comparator.comparingDouble(p -> p.get(1))).get().get(1));
            m.put("maxL", gt.stream().max(Comparator.comparingDouble(p -> p.get(1))).get().get(1));
        });
        return page;
    }

    @Override
    public boolean updateGroupSampleTagSingle(Integer groupId, Integer sampleId, String gk) {
        boolean update = this.update(new UpdateWrapper<GroupSampleRelation>()
                .eq("group_id", groupId)
                .eq("sample_id", sampleId).set("sample_tag1", gk));
        return update;
    }

    @Override
    public boolean deleteGroupSampleTagSingle(Integer groupId, Integer sampleId) {
        boolean remove = this.remove(new QueryWrapper<GroupSampleRelation>()
                .eq("group_id", groupId)
                .eq("sample_id", sampleId));
        return remove;
    }

    @Override
    public List<Map<String, Object>> getGroupCount(Integer groupId) {
        List<Map<String, Object>> mapList = baseMapper.selectMaps(new QueryWrapper<GroupSampleRelation>()
                .select("count(sample_tag1),sample_tag1 tag")
                .eq("group_id", groupId)
                .groupBy("sample_tag1"));
        return mapList;
    }

    @Override
    public boolean updateGroupSampleTag(Integer groupId, String oldTag, String tag) {
        boolean update = this.update(new UpdateWrapper<GroupSampleRelation>()
                .set("sample_tag1", tag)
                .eq("sample_tag1", oldTag)
                .eq("group_id",groupId));
        return update;
    }

    @Override
    public boolean deleteGroupSampleTag(Integer groupId, String tag) {
        boolean remove = this.remove(new QueryWrapper<GroupSampleRelation>()
                .eq("sample_tag1", tag)
                .eq("group_id", groupId));
        return remove;
    }
}
