package com.gct.dlview.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.core.result.Result;
import com.gct.dlview.entity.MachineResource;
import com.gct.dlview.entity.vo.PageBody;
import com.gct.dlview.mapper.MachineResourceMapper;
import com.gct.dlview.service.MachineResourceService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lloyd
 * @since 2021-08-13
 */
@Service
public class MachineResourceServiceImpl extends ServiceImpl<MachineResourceMapper, MachineResource> implements MachineResourceService {

    @Autowired
    MachineResourceMapper mapper;

    @Override
    public Result<List<MachineResource>> getListByCondition(MachineResource condition) {

        List<MachineResource> data = mapper.getListByCondition(condition);
        if (Objects.isNull(data) || data.isEmpty()) return Result.error("无数据");
        return Result.ok(data);
    }

    @Override
    public Result<List<MachineResource>> getPageByCondition(PageBody<MachineResource> pageBody) {
        if (Objects.isNull(pageBody) || Objects.isNull(pageBody.getPage()) || Objects.isNull(pageBody.getCondition()))
            return Result.error("无数据");
        Page<MachineResource> data = mapper.getListByCondition(pageBody.getPage(), pageBody.getCondition());
        if (Objects.isNull(data) || data.getRecords().isEmpty()) return Result.error("无数据");
        return Result.ok(data);
    }

    @Override
    public Result<List<Object>> getFiguredFilterByCondition(MachineResource condition, String columnName) {
        List<Map> re = mapper.getFilterByCondition(condition, columnName);
        System.out.println("re = " + re);
        if (Objects.isNull(re) || re.isEmpty()) return Result.error("无数据");
        List<Object> data = new ArrayList<>();
        re.forEach(x -> {
            if (Objects.nonNull(x) && Objects.nonNull(x.get(columnName)))
                data.add(x.get(columnName));
        });
        if (data.isEmpty()) return Result.error("无数据");
        return Result.ok(data);
    }

    @Override
    @Transactional
    public Result<Boolean> saveMachineResource(MachineResource data) {
        if (Objects.nonNull(data.getId()) && data.getId() <= 0) data.setId(null);
        if (Objects.isNull(data.getNetType()) || (data.getNetType() != 1 && data.getNetType() != 2))
            return Result.error("请设置正确的网络类型");

        if (Objects.isNull(data.getCreateTime())) data.setCreateTime(LocalDateTime.now());
        if (Objects.isNull(data.getUpdateTime())) data.setUpdateTime(LocalDateTime.now());
        try {
            boolean save = this.save(data);
            if (save) return Result.ok("保存成功");
            else return Result.error("保存失败");
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }
    }

    @Override
    public Result<Boolean> deleteOne(MachineResource data) {
        try {
            int i = mapper.deleteById(data.getId());
            if (i > 0) return Result.ok("删除成功");
            else return Result.error("删除失败");
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }
    }

    @Override
    public Result<Boolean> deleteBatch(List<MachineResource> data) {
        if (Objects.isNull(data) || data.isEmpty()) return Result.error("没有传入正确数据");
        List<Integer> ids = new ArrayList<>();
        data.forEach(x -> {
            ids.add(x.getId());
        });
        try {
            int i = mapper.deleteBatchIds(ids);
            if (i > 0) return Result.ok("删除成功");
            else return Result.error("删除失败");
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }
    }

    @Override
    public Result getOneByCondition(MachineResource machineResource) {
        List<MachineResource> data = mapper.getListByCondition(machineResource);
        if (Objects.isNull(data) || data.isEmpty()) return Result.error("无数据");

        return Result.ok(data.get(0));
    }
}
