package com.gct.dlview.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.util.SGTUtil;
import com.gct.dlview.entity.SampleDiagramCategoryIndex;
import com.gct.dlview.entity.SampleDiagramInterpretationTemp;
import com.gct.dlview.entity.SampleDiagramResource;
import com.gct.dlview.entity.SampleImportLog;
import com.gct.dlview.mapper.SampleDiagramResourceMapper;
import com.gct.dlview.service.SampleDiagramInterpretationTempService;
import com.gct.dlview.service.SampleDiagramResourceService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.*;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 功图样本源 服务实现类
 * </p>
 *
 * @author lloyd
 * @since 2021-08-15
 */
@Service
public class SampleDiagramResourceServiceImpl extends ServiceImpl<SampleDiagramResourceMapper, SampleDiagramResource> implements SampleDiagramResourceService {

    @Value("${sample-source-path}")
    private String sampleSourcePath;

    @Autowired
    SampleDiagramInterpretationTempService sampleDiagramInterpretationTempService;

    @Override
    public JSONObject getListByPage(String mark, String version, String batch, String gk, Integer impact, Integer curPage, Integer pageSize) {
        QueryWrapper<SampleDiagramResource> wrapper = new QueryWrapper<>();
        wrapper = wrapper.select("id,diagram_content,graphic_interpretation,sample_impact");
        if (mark != null && !"".equals(mark)) wrapper = wrapper.eq("import_mark", mark);
        if (version != null && !"".equals(version)) wrapper = wrapper.eq("data_version", version);
        if (batch != null && !"".equals(batch)) wrapper = wrapper.eq("batch_dec", batch);
        if (gk != null && !"".equals(gk)) wrapper = wrapper.eq("graphic_interpretation", gk);
        if (impact != null) wrapper = wrapper.eq("sample_impact", impact);
        wrapper = wrapper.orderByDesc("id");
        Page<SampleDiagramResource> page = baseMapper.selectPage(new Page<SampleDiagramResource>(curPage, pageSize), wrapper);
        List<JSONObject> sampleList = page.getRecords().stream().map(x -> {
            if (Objects.nonNull(x.getDiagramContent()) && !x.getDiagramContent().isEmpty()) {
                List<List<Double>> gt = SGTUtil.encodeToUseList(x.getDiagramContent());
                x.setDiagramContent(gt.toString());
                JSONObject jo = (JSONObject) JSON.toJSON(x);
                jo.put("minS", gt.stream().min(Comparator.comparingDouble(p -> p.get(0))).get().get(0));
                jo.put("maxS", gt.stream().max(Comparator.comparingDouble(p -> p.get(0))).get().get(0));
                jo.put("minL", gt.stream().min(Comparator.comparingDouble(p -> p.get(1))).get().get(1));
                jo.put("maxL", gt.stream().max(Comparator.comparingDouble(p -> p.get(1))).get().get(1));
                return jo;
            }
            return new JSONObject();
        }).collect(Collectors.toList());
        JSONObject res = (JSONObject)JSON.toJSON(page);
        res.put("records",sampleList);
        return res;
    }

    @Override
    public List<SampleDiagramResource> getList(String mark, String version, String batch, String gk, Integer impact) {
        QueryWrapper<SampleDiagramResource> wrapper = new QueryWrapper<>();
        wrapper = wrapper.select("id,diagram_content,graphic_interpretation");
        if (mark != null && !"".equals(mark)) wrapper = wrapper.eq("import_mark", mark);
        if (version != null && !"".equals(version)) wrapper = wrapper.eq("data_version", version);
        if (batch != null && !"".equals(batch)) wrapper = wrapper.eq("batch_dec", batch);
        if (gk != null && !"".equals(gk)) wrapper = wrapper.eq("graphic_interpretation", gk);
        if (impact != null) wrapper = wrapper.eq("sample_impact", impact);
        return baseMapper.selectList(wrapper);
    }

    @Override
    public void dealSampleFile(MultipartFile multipartFile, String mark, String version, String batch) throws IOException {
        String username = SecurityContextHolder.getContext().getAuthentication().getName();
        File dir = new File(sampleSourcePath);
        if (!dir.exists()) dir.mkdirs();
        File file = new File(sampleSourcePath + multipartFile.getOriginalFilename());
        multipartFile.transferTo(file);
        BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(file)));
        String line;
        String transactionId = UUID.randomUUID().toString();
        System.out.println(transactionId);
        int sampleCount = 0;
        while ((line = br.readLine()) != null) {
            JSONObject jo = JSONObject.parseObject(new String(Base64.getDecoder().decode(line.getBytes())));
            System.out.println(jo.toString());
            String sgt = jo.getString("sgt");
            String gk = jo.getString("gk");
            String cgk = jo.getString("cgk");
            SampleDiagramResource sampleDiagramResource = new SampleDiagramResource();
            sampleDiagramResource.setCreateTime(LocalDateTime.now());
            sampleDiagramResource.setDiagramContent(SGTUtil.encodeToString(JSONObject.parseObject(sgt, double[][].class)));
            sampleDiagramResource.setGraphicInterpretation(gk);
            sampleDiagramResource.setRealPerformance(cgk);
            sampleDiagramResource.setImportMark(mark);
            sampleDiagramResource.setDataVersion(version);
            sampleDiagramResource.setBatchDec(batch);
            sampleDiagramResource.setIsAvailable(1);
            sampleDiagramResource.setImportTransactionId(transactionId);
            sampleDiagramResource.setSampleImpact(gk.equals(cgk) ? 1 : 0);
            sampleDiagramResource.insert();
            sampleCount++;
        }
        SampleImportLog sampleImportLog = new SampleImportLog();
        sampleImportLog.setId(0);
        sampleImportLog.setCreateTime(LocalDateTime.now());
        sampleImportLog.setSampleType(1);
        sampleImportLog.setSampleSize(BigDecimal.valueOf(sampleCount));
        sampleImportLog.setImportCompletedSize(BigDecimal.valueOf(sampleCount));
        sampleImportLog.setImportFailedSize(BigDecimal.ZERO);
        sampleImportLog.setProcessState(2);
        sampleImportLog.setTempLocalPath(file.getPath());
        sampleImportLog.setFileSize(BigDecimal.valueOf(multipartFile.getSize()));
        sampleImportLog.setCreateUser(username);
        sampleImportLog.setImportTransactionId(transactionId);
        sampleImportLog.setImportSystemVersion("v1.0");
        sampleImportLog.setImportSystemMethod("个人导入");
        sampleImportLog.insert();

        SampleDiagramCategoryIndex sampleDiagramCategoryIndex = new SampleDiagramCategoryIndex();
        sampleDiagramCategoryIndex.setBatch(batch);
        sampleDiagramCategoryIndex.setMark(mark);
        sampleDiagramCategoryIndex.setVersion(version);
        SampleDiagramCategoryIndex one = sampleDiagramCategoryIndex.selectOne(new QueryWrapper<SampleDiagramCategoryIndex>()
                .eq("batch", batch)
                .eq("mark", mark).eq("version", version));
        if (one==null)sampleDiagramCategoryIndex.insert();
    }

    @Override
    public List<Map<String,Object>> getGkCount(String mark, String version, String batch, String gk, Integer impact) {
        QueryWrapper<SampleDiagramResource> wrapper = new QueryWrapper<>();
        wrapper = wrapper.select("count(*),graphic_interpretation \"name\"");
        if (mark != null && !"".equals(mark)) wrapper = wrapper.eq("import_mark", mark);
        if (version != null && !"".equals(version)) wrapper = wrapper.eq("data_version", version);
        if (batch != null && !"".equals(batch)) wrapper = wrapper.eq("batch_dec", batch);
        if (gk != null && !"".equals(gk)) wrapper = wrapper.eq("graphic_interpretation", gk);
        if (impact != null) wrapper = wrapper.eq("sample_impact", impact);
        wrapper.last("group by name");
        return baseMapper.selectMaps(wrapper);
    }

    @Override
    public boolean updateSampleGk(String mark, String version, String batch, Integer impact, String oldGk, String gk) {
        UpdateWrapper<SampleDiagramResource> wrapper = new UpdateWrapper<>();
        wrapper = wrapper.set("graphic_interpretation",gk);
        if (mark != null && !"".equals(mark)) wrapper = wrapper.eq("import_mark", mark);
        if (version != null && !"".equals(version)) wrapper = wrapper.eq("data_version", version);
        if (batch != null && !"".equals(batch)) wrapper = wrapper.eq("batch_dec", batch);
        if (oldGk != null && !"".equals(oldGk)) wrapper = wrapper.eq("graphic_interpretation", oldGk);
        if (impact != null) wrapper = wrapper.eq("sample_impact", impact);
        boolean update = this.update(wrapper);
        if(update)sampleDiagramInterpretationTempService.saveOrUpdate(
               new SampleDiagramInterpretationTemp().setInterpretation(gk)
                ,new QueryWrapper<SampleDiagramInterpretationTemp>()
                        .eq("interpretation",gk)
        );
        return update;
    }

    @Override
    public boolean deleteSample(String mark, String version, String batch, Integer impact, String gk) {
        QueryWrapper<SampleDiagramResource> wrapper = new QueryWrapper<>();
        wrapper = wrapper.select("count(*),graphic_interpretation \"name\"");
        if (mark != null && !"".equals(mark)) wrapper = wrapper.eq("import_mark", mark);
        if (version != null && !"".equals(version)) wrapper = wrapper.eq("data_version", version);
        if (batch != null && !"".equals(batch)) wrapper = wrapper.eq("batch_dec", batch);
        if (gk != null && !"".equals(gk)) wrapper = wrapper.eq("graphic_interpretation", gk);
        if (impact != null) wrapper = wrapper.eq("sample_impact", impact);
        return this.remove(wrapper);
    }

    @Override
    public boolean updateSampleGkSingle(Integer id, String oldGk, String gk) {
        UpdateWrapper<SampleDiagramResource> wrapper = new UpdateWrapper<>();
        wrapper = wrapper.set("graphic_interpretation",gk);
        wrapper = wrapper.eq("id",id);
        boolean update = this.update(wrapper);
        if(update)sampleDiagramInterpretationTempService.saveOrUpdate(
                new SampleDiagramInterpretationTemp().setInterpretation(gk)
                ,new QueryWrapper<SampleDiagramInterpretationTemp>()
                        .eq("interpretation",gk)
        );
        return update;
    }

    @Override
    public boolean updateSampleImpact(Integer id, Integer sampleImpact) {
        SampleDiagramResource sampleDiagramResource = new SampleDiagramResource().setId(id).setSampleImpact(1 - sampleImpact);
        return baseMapper.updateById(sampleDiagramResource)>0;
    }
}
