package com.gct.dlview.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.core.result.Result;
import com.gct.dlview.entity.GroupSampleRelationIndex;
import com.gct.dlview.entity.TaskStateEnum;
import com.gct.dlview.entity.TrainTask;
import com.gct.dlview.mapper.TrainTaskMapper;
import com.gct.dlview.service.GroupSampleRelationIndexService;
import com.gct.dlview.service.TrainTaskService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author xsr
 * @since 2021-09-08
 */
@Service
public class TrainTaskServiceImpl extends ServiceImpl<TrainTaskMapper, TrainTask> implements TrainTaskService {

    @Autowired
    GroupSampleRelationIndexService groupSampleRelationIndexService;

    @Override
    public JSONObject getTrainTask(Integer curPage, Integer pageSize) {
        Page<Map<String, Object>> trainTask = baseMapper.getTrainTask(new Page<>(curPage, pageSize));
        List<JSONObject> res = trainTask.getRecords().stream().map(m -> {
            JSONObject jo = new JSONObject();
            jo.put("id", m.get("id"));
            jo.put("trainServerId", m.get("train_server_id"));
            jo.put("sampleSourceId", m.get("group_id"));
            jo.put("sampleSource", m.get("group_name"));
            jo.put("sampleSourceVersion", m.get("train_sample_source_version_code"));
            jo.put("trainServer", m.get("server_name"));
            Integer taskStateControl = Integer.parseInt(m.get("task_state_control").toString());
            String taskState = TaskStateEnum.getById(Integer.parseInt(m.get("task_state").toString())).getDec();
            if (taskStateControl != 0) taskState = TaskStateEnum.getById(taskStateControl).getDec();
            jo.put("isStable", taskStateControl == 0);
            jo.put("taskState", taskState);
            jo.put("curStep", m.get("task_step_info"));
            jo.put("enable", m.get("task_enable").equals(1));
            jo.put("taskSchedule", m.get("progress_percentage"));
            jo.put("enableAutoStart", m.get("enable_auto_start").equals(1));
            jo.put("enableAutoLoadBalance", m.get("enable_auto_load_balance").equals(1));
            jo.put("enableStepWait", m.get("enable_step_wait").equals(1));
            jo.put("createTime", m.get("create_time").toString().substring(0, 19));
            jo.put("updateTime", m.get("update_time").toString().substring(0, 19));
            jo.put("log", m.get("execute_log"));
            return jo;
        }).collect(Collectors.toList());
        JSONObject page = (JSONObject) JSON.toJSON(trainTask);
        page.put("records", res);
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Result saveOrUpdateTrainTask(Integer id, Integer groupId, Integer groupVersion, Integer serverId, boolean enableAutoLoadBalance, boolean enableAutoStart, boolean enableStepWait) {
        TrainTask trainTask = new TrainTask();
        trainTask.setId(id);
        trainTask.setTrainSampleSourceId(groupId);
        trainTask.setTrainSampleSourceVersionCode(groupVersion);
        //如果为最新版本则锁定，创建新版本
        Object maxVersion = groupSampleRelationIndexService.getMap(
                new QueryWrapper<GroupSampleRelationIndex>().select("max(version_serial_code)").eq("group_id", groupId)).get("max");
        if (groupVersion.equals(maxVersion)) groupSampleRelationIndexService.lockLastVersion(groupId, groupVersion);
        trainTask.setTrainServerId(serverId);
        trainTask.setEnableAutoLoadBalance(enableAutoLoadBalance ? 1 : 0);
        trainTask.setEnableAutoStart(enableAutoStart ? 1 : 0);
        trainTask.setEnableStepWait(enableStepWait ? 1 : 0);
        trainTask.setUpdateTime(LocalDateTime.now());
        if (trainTask.getId() == -1) {
            trainTask.setCreateTime(LocalDateTime.now());
            trainTask.setTaskStepCode(0);
            trainTask.setProgressPercentage(BigDecimal.ZERO);
            trainTask.setTaskEnable(0);
            trainTask.setTaskStateControl(0);
            trainTask.setTaskStepInfo("无");
            trainTask.setTaskState(TaskStateEnum.INIT.getId());
        }
        return trainTask.getId() == -1 ? (baseMapper.insert(trainTask) > 0 ? Result.ok("创建成功") : Result.error("创建失败"))
                : (baseMapper.updateById(trainTask) > 0 ? Result.ok("更新成功") : Result.error("更新失败"));
    }

    @Override
    public Result deleteTrainTask(Integer id) {
        return baseMapper.deleteById(id) > 0 ? Result.ok("删除成功") : Result.ok("删除失败");
    }

    @Override
    public HashMap<Serializable, TrainTask> getMapAllEnable() {
        List<TrainTask> lis = getListAllEnable();
        return lis.stream().collect(Collectors.toMap(TrainTask::getId, x -> x, (a, b) -> b, HashMap::new));
    }

    @Override
    public List<TrainTask> getListAllEnable() {
        TrainTask condition = new TrainTask();
        condition.setTaskEnable(1);
        List<TrainTask> lis = baseMapper.getListByCondition(condition);
        if (Objects.isNull(lis) || lis.isEmpty()) return new ArrayList<>();
        return lis;
    }

    @Override
    public Result enableTrainTask(Integer id) {
        return baseMapper.update(null,
                new UpdateWrapper<TrainTask>()
                        .set("task_enable", 1)
                        .set("update_time", LocalDateTime.now()).
                        eq("id", id)) > 0
                ? Result.ok("启用成功") : Result.error("启用失败");
    }

    @Override
    public JSONArray getReachableStateList(String taskState) {
        return TaskStateEnum.getReachableStateList(TaskStateEnum.getByDec(taskState));
    }

    @Override
    public Result changeTrainTaskState(String curTaskState, Integer id, Integer taskStateControl) {
        return baseMapper.update(null,
                new UpdateWrapper<TrainTask>()
                        .set("task_state_control", taskStateControl)
                        .eq("task_state", TaskStateEnum.getByDec(curTaskState).getId())
                        .eq("id", id)
        )>0?Result.ok("控制成功"):Result.error("控制失败");
    }
}
