package com.gct.dlview.thrift;

import com.alibaba.fastjson.JSONObject;
import com.gct.dlview.config.ApplicationEventPublisher;
import com.gct.dlview.entity.TaskStateEnum;
import com.gct.dlview.entity.TrainTask;
import lombok.Data;

import java.time.LocalDateTime;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadLocalRandom;
import java.util.concurrent.TimeUnit;

import static com.gct.dlview.entity.TaskStateEnum.*;

@Data
public class DLTrainTask implements DLTask {

    private TrainTask trainTaskCopy;

    private UUID remoteControlStamp;

    /**
     * notice:not implements deep clone
     *
     * @param trainTask
     */
    public DLTrainTask accessFromTaskStorage(TrainTask trainTask) {
        if (Objects.isNull(trainTaskCopy)) {
            trainTaskCopy = trainTask;
            if (trainTaskCopy.getTaskStateControl() > 0) remoteControlStamp = UUID.randomUUID();
        } else {
            //运行任务中 仅支持修改控制状态，其它修改不被保留,控制不可以被撤销，之前有控制没有执行完成后续的控制修改无效
            if (trainTask.getTaskStateControl() == 0 || trainTask.getTaskStateControl() == trainTaskCopy.getTaskStateControl())
                return this;
            if (Objects.nonNull(remoteControlStamp)) return this;
            trainTaskCopy.setTaskStateControl(trainTask.getTaskStateControl());
            remoteControlStamp = UUID.randomUUID();
        }

        asyncPushTaskUpdateEvent();
        return this;
    }

    public DLTrainTask accessFromSyncQueue(DLTrainTask dlTrainTask) {
        Integer old = trainTaskCopy.getTaskStateControl();
        this.trainTaskCopy = dlTrainTask.getTrainTaskCopy();
        if ((this.remoteControlStamp != null || dlTrainTask.getRemoteControlStamp() != null)
                && (remoteControlStamp == null || dlTrainTask.getRemoteControlStamp() == null
                || !this.remoteControlStamp.equals(dlTrainTask.getRemoteControlStamp())))
            this.trainTaskCopy.setTaskStateControl(old);
        trainTaskCopy.setUpdateTime(LocalDateTime.now());
        asyncPushTaskUpdateEvent();
        return this;
        //控制持久化到库里才完成
//        if(remoteControlStamp!=null&&trainTaskCopy.getTaskStateControl()==0) remoteControlStamp=null;
    }

    private void asyncPushTaskUpdateEvent() {
        DLTrainTask dlTrainTask = this.clone();
        CompletableFuture<Void> voidRe = CompletableFuture.runAsync(() -> {
            if (Objects.isNull(dlTrainTask)) return;
            TrainTask actualTask = dlTrainTask.getTrainTaskCopy();
            if (Objects.isNull(actualTask)) return;

            //任务处于稳定状态
            if (isStable(actualTask.getTaskState()))
                //且没有控制
                if (actualTask.getTaskStateControl() == 0)
                    return;//不处理
            try {
                TimeUnit.SECONDS.sleep(ThreadLocalRandom.current().nextLong(1, 5));//减少瞬时并发量
            } catch (InterruptedException ignore) {
            }
            ApplicationEventPublisher.publishEvent(new PushTrainTaskEvent(dlTrainTask));
        });

    }

    public TrainTask saveToTaskStorage() {
        TrainTask save = new TrainTask();
        save.setTaskEnable(trainTaskCopy.getTaskEnable());
        save.setTaskState(trainTaskCopy.getTaskState());
        save.setCreateTime(trainTaskCopy.getCreateTime());
        save.setEnableAutoStart(trainTaskCopy.getEnableAutoStart());
        save.setEnableAutoLoadBalance(trainTaskCopy.getEnableAutoLoadBalance());
        save.setEnableStepWait(trainTaskCopy.getEnableStepWait());
        save.setExecuteLog(trainTaskCopy.getExecuteLog());
        save.setId(trainTaskCopy.getId());
        save.setIntermediateData(trainTaskCopy.getIntermediateData());
        save.setProgressPercentage(trainTaskCopy.getProgressPercentage());
        save.setTaskStepCode(trainTaskCopy.getTaskStepCode());
        save.setUpdateTime(trainTaskCopy.getUpdateTime());
        save.setTaskStepInfo(trainTaskCopy.getTaskStepInfo());
        save.setTrainSampleSourceId(trainTaskCopy.getTrainSampleSourceId());
        save.setTrainServerId(trainTaskCopy.getTrainServerId());
        save.setTrainSampleSourceVersionCode(trainTaskCopy.getTrainSampleSourceVersionCode());

        if (remoteControlStamp != null && trainTaskCopy.getTaskStateControl() == 0) {
            remoteControlStamp = null;
            save.setTaskStateControl(trainTaskCopy.getTaskStateControl());
        }
        return save;
    }

    @Override
    public DLTrainTask clone() {
        return JSONObject.parseObject(JSONObject.toJSONString(this), this.getClass());
    }
}
