package com.gct.dlview.thrift;

import com.alibaba.fastjson.JSONObject;
import com.gct.dlview.entity.DeepLearnTrainServer;
import com.gct.dlview.entity.TrainTask;
import com.gct.dlview.service.DeepLearnTrainServerService;
import com.gct.dlview.service.MachineResourceService;
import com.gct.dlview.service.TrainTaskService;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Component
@Slf4j
public class JDBCPersistProvider implements PersistProvider {
    @Autowired
    MachineResourceService machineResourceService;

    @Autowired
    DeepLearnTrainServerService deepLearnTrainServerService;

    @Autowired
    TrainTaskService trainTaskService;

    @Override
    public void updateRunState(ServerConfig server) {
        boolean b = deepLearnTrainServerService.updateRunStateById(server.getRunState(), server.getId());
        if (!b) log.error("Persist failed, server={}", server);
    }

    @Override
    public void updateRunStateBatch(HashMap<Serializable, ServerConfig> serverConfigs) {
        HashMap<Serializable, DeepLearnTrainServer> data = deepLearnTrainServerService.getMapByIds(serverConfigs.keySet().stream().collect(Collectors.toList()));
        if (Objects.isNull(data)) {
            log.warn("Persist failed,can not read data,retry as soon");
            return;
        }
        //更新 only 不检查 差异更新 memory provider控制该逻辑
        data.forEach((key, value) -> {
            ServerConfig serverConfig = serverConfigs.get(key);
            if (Objects.nonNull(serverConfig)) {
                value.setRunState(serverConfig.getRunState());
            }
        });
        try{
            boolean b = deepLearnTrainServerService.updateBatchById(data.values());
            if (!b) log.error("Persist batch failed,data:{}", JSONObject.toJSONString(data.values()));
        }catch (Exception ignore){
            log.error("Persist batch net failed,data:{}", JSONObject.toJSONString(data.values()));
        }
    }

    @Override
    public void updateTrainTaskBatch(List<TrainTask> trainTask) {
        try{
            boolean b = trainTaskService.updateBatchById(trainTask);
            if (!b) log.error("Persist batch failed,data:{}", JSONObject.toJSONString(trainTask));
        }catch (Exception ignore){
            log.error("Persist batch net failed,data:{}", JSONObject.toJSONString(trainTask));
        }
    }

    @Override
    public HashMap<Serializable, ServerConfig> getAllConfig() {
        List<DeepLearnTrainServer> lis = deepLearnTrainServerService.getAllList();
        if (Objects.isNull(lis)) return null;
        HashMap<Serializable, ServerConfig> m = new HashMap<>();
        lis.forEach(item -> {

            if (Objects.isNull(item)) {
                LoggerFactory.getLogger(this.getClass()).warn("Need to provide server config correctly,this:null");
                return;
            }
            if (Objects.isNull(item.getId())) {
                LoggerFactory.getLogger(this.getClass()).warn("Need to provide server config correctly,this:{}", JSONObject.toJSONString(item));
                return;
            }
            m.put(item.getId(), new ServerConfig() {
                {
                    setId(item.getId());
                    if (Objects.nonNull(item.getIpAddr()))
                        setHost(item.getIpAddr());
                    if (Objects.nonNull(item.getTimeout()))
                        setTimout(item.getTimeout());
                    if (Objects.nonNull(item.getRunState()))
                        setRunState(item.getRunState());
                    if (Objects.nonNull(item.getServerName()))
                        setName(item.getServerName());
                    if (Objects.nonNull(item.getPort()))
                        setPort(item.getPort());
                    if (Objects.nonNull(item.getLocalOriginPath()))
                        setLocalOriginPath(item.getLocalOriginPath());
                }
            });

        });
        return m;
    }


    @Override
    public HashMap<Serializable, TrainTask> getAllTrainTask() {
//        List<TrainTask> lis = trainTaskService.getListAllEnable();
//        if (Objects.isNull(lis)) return null;
//        HashMap<Serializable, DLTrainTask> m = new HashMap<>();
//        lis.forEach(item -> {
//            if (Objects.isNull(item) || Objects.isNull(item.getId()) || item.getId() <= 0) {
//                log.warn("Need to provide task detail correctly,this:{}", JSONObject.toJSONString(item));
//                return;
//            }
//            m.put(item.getId(), new DLTrainTask().accessFromTaskStorage(item));
//        });
//        return m;

        return trainTaskService.getMapAllEnable();
    }
}
