package com.gct.dlview.thrift.job;

import org.quartz.*;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

@Configuration
@EnableConfigurationProperties(JobConfigProperties.class)
@ConditionalOnProperty(prefix = "gct.thrift.register-center", name = "enabled", havingValue = "true")
public class JobConfig {

    private JobConfigProperties properties;

    public JobConfig(JobConfigProperties properties) {
        this.properties = properties;
    }

    private static JobKey heartBeatJobKey = new JobKey("heartBeatJob");
    private static JobKey persistJobKey = new JobKey("persistJobKey");
    private static JobKey persist2JobKey = new JobKey("persist2JobKey");
    private static JobKey syncCacheJobKey = new JobKey("syncCacheJobKey");
    private static JobKey syncCacheJobKey2 = new JobKey("syncCacheJobKey2");


    @Bean
    public JobDetail heartBeatJob() {
        return JobBuilder.newJob(HeartBeatWorker.class).withIdentity(heartBeatJobKey).storeDurably().build();
    }

    @Bean
    public Trigger heartBeatJobTrigger() {
        SimpleScheduleBuilder scheduleBuilder = SimpleScheduleBuilder.simpleSchedule().withIntervalInMilliseconds(properties.getHeartInterval()).repeatForever();
        return TriggerBuilder.newTrigger().forJob(heartBeatJobKey).startNow().withSchedule(scheduleBuilder).build();
    }


    @Bean
    public JobDetail persistJob() {
        return JobBuilder.newJob(PersistWorker.class).withIdentity(persistJobKey).storeDurably().build();
    }

    @Bean
    public Trigger persistJobTrigger() {
        SimpleScheduleBuilder scheduleBuilder = SimpleScheduleBuilder.simpleSchedule().withIntervalInMilliseconds(properties.getPersistConfigInterval()).repeatForever();
        return TriggerBuilder.newTrigger().forJob(persistJobKey).startNow().withSchedule(scheduleBuilder).build();
    }

    @Bean
    public JobDetail persist2Job() {
        return JobBuilder.newJob(Persist2Worker.class).withIdentity(persist2JobKey).storeDurably().build();
    }

    @Bean
    public Trigger persist2JobTrigger() {
        SimpleScheduleBuilder scheduleBuilder = SimpleScheduleBuilder.simpleSchedule().withIntervalInMilliseconds(properties.getPersistTaskInterval()).repeatForever();
        return TriggerBuilder.newTrigger().forJob(persist2JobKey).startNow().withSchedule(scheduleBuilder).build();
    }

    @Bean
    public JobDetail syncCacheJob() {
        return JobBuilder.newJob(SyncCacheWorker.class).withIdentity(syncCacheJobKey).storeDurably().build();
    }

    @Bean
    public Trigger syncCacheJobTrigger() {
        SimpleScheduleBuilder scheduleBuilder = SimpleScheduleBuilder.simpleSchedule().withIntervalInMilliseconds(properties.getSyncConfigCacheInterval()).repeatForever();
        return TriggerBuilder.newTrigger().forJob(syncCacheJobKey).startNow().withSchedule(scheduleBuilder).build();
    }

    @Bean
    public JobDetail syncCacheJob2() {
        return JobBuilder.newJob(SyncCache2Worker.class).withIdentity(syncCacheJobKey2).storeDurably().build();
    }

    @Bean
    public Trigger syncCacheJob2Trigger() {
        SimpleScheduleBuilder scheduleBuilder = SimpleScheduleBuilder.simpleSchedule().withIntervalInMilliseconds(properties.getPersistTaskInterval()).repeatForever();
        return TriggerBuilder.newTrigger().forJob(syncCacheJobKey2).startNow().withSchedule(scheduleBuilder).build();
    }


}
