package com.gct.dlview.thrift.train;

import com.gct.dlview.thrift.TrainTaskExecute;
import lombok.extern.slf4j.Slf4j;
import org.apache.thrift.TProcessor;
import org.apache.thrift.server.TServer;
import org.apache.thrift.server.TThreadPoolServer;
import org.apache.thrift.transport.TServerSocket;
import org.apache.thrift.transport.TServerTransport;

/**
 * dec:
 * <p>
 * TrainLogServer
 * </p>
 *
 * @author lloyd
 * @version 1.0.1
 * @since 2021/11/10 下午6:35
 */
@Slf4j
public class TrainLogServer {

    private TrainTaskExecute execute;

    private TrainProperties properties;

    public TrainLogServer(TrainTaskExecute execute, TrainProperties properties) {
        this.execute = execute;
        this.properties = properties;
    }

    public void init() {
        new Thread(() -> {
            Thread.currentThread().setName("TrainLogServer");
            if (!properties.isEnabled()) return;
            try {
                TrainServerWrapper handler = new TrainServerWrapper(execute);

                TServerTransport serverTransport = new TServerSocket(properties.getPort());

                // Use this for a multithreaded server
                TProcessor processor = new TrainServer.Processor(handler);
                TServer server = new TThreadPoolServer(new TThreadPoolServer.Args(serverTransport).processor(processor));

                log.info("\033[1;30mStart the train log server,with port->{}\033[0m ", properties.getPort());
                server.serve();
            } catch (Exception e) {
                log.info("\033[1;31mThe train log server start failed,cause by: {}\033[0m ", e.getMessage());
            }
        }).start();

    }
}
