package com.gct.dlview.thrift.train;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * dec:
 * <p>
 * TrainParam
 * </p>
 *
 * @author lloyd
 * @version 1.0.1
 * @since 2021/11/5 下午3:20
 */
public class TrainParam {

    /**
     * 分页传输样本<br>
     * 当前页
     */
    int curPage;
    /**
     * 分页传输样本<br>
     * 页大小配置
     */
    int pageSize;
    /**
     * 分页传输样本<br>
     * 总页数
     */
    int pageTotal;

    /**
     * 状态控制器从0开始，所以为了记忆方便这里也从0开始<br>
     * 步骤一操作状态位<br>
     * 用于异步信号<br>
     * 分页发送异步状态：0表示初始状态，1开始发送，2完成所有页发送，3表示异常中断
     * 有c++端根据page信息做幂等，解决同一页重复发送时的问题
     */
    int statStep0;
    /**
     * 状态控制器从0开始，所以为了记忆方便这里也从0开始<br>
     * 步骤二操作状态位<br>
     * 用于异步信号<br>
     * 该步状态：0表示初始状态，1表示开始，2表示完成，3表示异常中断
     */
    int statStep1;
    /**
     * 状态控制器从0开始，所以为了记忆方便这里也从0开始<br>
     * 步骤三操作状态位<br>
     * 用于异步信号<br>
     * 该步状态：0表示初始状态，1表示开始，2表示完成，3表示异常中断
     */
    int statStep2;
    /**
     * 状态控制器从0开始，所以为了记忆方便这里也从0开始<br>
     * 步骤四操作状态位<br>
     * 用于异步信号<br>
     * 该步状态：0表示初始状态，1表示开始，2表示完成，3表示异常中断
     */
    int statStep3;
    /**
     * 状态控制器从0开始，所以为了记忆方便这里也从0开始<br>
     * 步骤五操作状态位<br>
     * 用于异步信号<br>
     * 该步状态：0表示初始状态，1表示开始，2表示完成，3表示异常中断
     */
    int statStep4;

    /**
     * 状态控制器从0开始，所以为了记忆方便这里也从0开始<br>
     * 步骤六操作状态位<br>
     * 用于异步信号<br>
     * 没有用到
     */
    int statStep5;

    /**
     * 状态控制器从0开始，所以为了记忆方便这里也从0开始<br>
     * 步骤七操作状态位<br>
     * 用于异步信号<br>
     * 没有用到
     */
    int statStep6;

    /**
     * 样本临时存储文件
     */
    String path1;
    /**
     * 标准化临时存储文件
     */
    String path2;
    /**
     * 特征解析临时存储文件
     */
    String path3;
    /**
     * 训练出的模型存储文件
     */
    String path4;

    private TrainParam() {
    }

    public static TrainParam init() {
        TrainParam t = new TrainParam();
        t.curPage = 0;
        t.pageSize = 20;
        t.pageTotal = 0;
        t.statStep0 = 0;
        t.statStep1 = 0;
        t.statStep2 = 0;
        t.statStep3 = 0;
        t.statStep4 = 0;
        t.statStep5 = 0;
        t.statStep6 = 0;
        t.path1 = UUID.randomUUID().toString().replace("-", "") + ".samp";
        t.path2 = UUID.randomUUID().toString().replace("-", "") + ".rex";
        t.path3 = UUID.randomUUID().toString().replace("-", "") + ".hog";
        t.path4 = UUID.randomUUID().toString().replace("-", "") + ".model";
        return t;
    }


    public static TrainParam trans(String json) {
        try {
            return JSONObject.parseObject(json, TrainParam.class);
        } catch (Exception ignore) {
            return null;
        }
    }

    public String json() {
        return JSONObject.toJSONString(this);
    }

    public List<String> values() {
        return JSON.parseObject(this.json()).values().stream().map(Object::toString).collect(Collectors.toList());
    }

    public static TrainParam trans(List<String> params) throws Exception {
        try {
            TrainParam t = new TrainParam();
            t.curPage = Integer.parseInt(params.get(0));
            t.pageSize = Integer.parseInt(params.get(1));
            t.pageTotal = Integer.parseInt(params.get(2));
            t.statStep0 = Integer.parseInt(params.get(3));
            t.statStep1 = Integer.parseInt(params.get(4));
            t.statStep2 = Integer.parseInt(params.get(5));
            t.statStep3 = Integer.parseInt(params.get(6));
            t.statStep4 = Integer.parseInt(params.get(7));
            t.statStep5 = Integer.parseInt(params.get(8));
            t.statStep6 = Integer.parseInt(params.get(9));
            t.path1 = params.get(10);
            t.path2 = params.get(11);
            t.path3 = params.get(12);
            t.path4 = params.get(13);
            return t;
        } catch (Exception e) {
            throw e;
        }
    }
}
