package com.gct.dlview.thrift.train;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.dlview.entity.TrainTask;
import com.gct.dlview.service.GroupSampleRelationService;
import com.gct.dlview.service.SampleDiagramInterpretationTempService;
import com.gct.dlview.thrift.DLTrainTask;
import com.gct.dlview.thrift.TaskExecuteStep;
import com.gct.dlview.thrift.TaskState;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * dec:
 * <p>
 * TrainStepOne
 * </p>
 *
 * @author lloyd
 * @version 1.0.1
 * @since 2021/11/5 下午5:41
 */
public class TrainStep1 implements TaskExecuteStep<DLTrainTask, TrainServerWrapper> {

    //region private
    private GroupSampleRelationService groupSampleRelationService;
    private SampleDiagramInterpretationTempService sampleDiagramInterpretationTempService;
    //endregion

    /**
     * Construction
     *
     * @param groupSampleRelationService             It's necessary.
     * @param sampleDiagramInterpretationTempService It's necessary.
     */
    public TrainStep1(GroupSampleRelationService groupSampleRelationService, SampleDiagramInterpretationTempService sampleDiagramInterpretationTempService) {
        this.groupSampleRelationService = groupSampleRelationService;
        this.sampleDiagramInterpretationTempService = sampleDiagramInterpretationTempService;
    }

    @Override
    public boolean execute(DLTrainTask task) {
        TrainServerWrapper server = (TrainServerWrapper) serverStorage.get();

        TrainTask actualTaskInfo = task.getTrainTaskCopy();

        TrainParam param = TrainParam.trans(actualTaskInfo.getIntermediateData());
        if (Objects.isNull(param)) return true;//参数延迟情况截断，减少c端幂等复杂度
        if (param.pageTotal != 0 && param.curPage + 1 <= param.pageTotal)
            param.curPage += 1;
        Page<Map<String, Object>> page = groupSampleRelationService.getGroupSample(actualTaskInfo.getTrainSampleSourceId(), null, actualTaskInfo.getTrainSampleSourceVersionCode(), param.curPage, param.pageSize);
        if (param.pageTotal == 0) {
//            server.setSampleCount((int) page.getTotal());
            param.pageTotal = (int) page.getPages();
        }
        Map<String, Integer> interpretationNameIdMap = sampleDiagramInterpretationTempService.getInterpretationNameIdMap();
        List<List<List<Double>>> sampleList = page.getRecords().stream().map(m -> (List<List<Double>>) m.get("diagramContent")).collect(Collectors.toList());
        List<Integer> tags = page.getRecords().stream().map(m -> Integer.parseInt(interpretationNameIdMap.get(m.get("graphicInterpretation")).toString())).collect(Collectors.toList());

        server.transSGTSamplePage(actualTaskInfo.getId().toString(), sampleList, tags, param.curPage, param.pageSize, param.pageTotal);
        //此处参数不必要，由c端读回
//        actualTaskInfo.setIntermediateData(param.json());
        return true;
    }
}
