package com.gct.dlview.thrift.train;

import com.gct.dlview.entity.TaskStateEnum;
import com.gct.dlview.entity.TrainTask;
import com.gct.dlview.thrift.DLTrainTask;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Objects;

import static com.gct.dlview.entity.TaskStateEnum.*;
import static com.gct.dlview.entity.TaskStateEnum.INTERRUPT;

/**
 * dec:
 * <p>
 * TrainStepController
 * </p>
 *
 * @author lloyd
 * @version 1.0.1
 * @since 2021/11/8 上午9:34
 */
@Component
@Slf4j
public class TrainStepController {


    public int checkControlSignal(DLTrainTask task, TrainServerWrapper server) {
        TrainTask actualTaskInfo = task.getTrainTaskCopy();
        if (actualTaskInfo.getTaskEnable() == 0) return -1;
        //初始状态且没有设置自启动，将默认处于关闭状态
        if (actualTaskInfo.getTaskState() == 0 && actualTaskInfo.getEnableAutoStart() == 2 && actualTaskInfo.getTaskStateControl() == 0) {
            actualTaskInfo.setTaskState(CLOSED.getId());
            return -1;
        }

        if (actualTaskInfo.getTaskStateControl() > 0)
            switch (TaskStateEnum.getById(actualTaskInfo.getTaskStateControl())) {//前端做了状态判断，这里不需要
                case PAUSING:
                    if (server.isSafetyPoint(actualTaskInfo.getId().toString())) {
                        actualTaskInfo.setTaskState(PAUSED.getId());
                        actualTaskInfo.setTaskStateControl(0);
                    }
                    return -1;
                case CONTINUE:
                    TrainParam param = TrainParam.trans(actualTaskInfo.getIntermediateData());
                    if (Objects.isNull(param)) {
                        log.error("Unknown error happened on 'CONTINUE' control,the train param is incorrect or step controller run with wrong order");
                        actualTaskInfo.setTaskState(INTERRUPT.getId());
                        return -1;
                    }
                    if (server.setProcessParam(actualTaskInfo.getId().toString(), param.values())) {
                        actualTaskInfo.setTaskState(RUNNING.getId());
                        actualTaskInfo.setTaskStateControl(0);
                        return checkControlSignal(task, server);
                    }
                    return -1;
                case CLOSING:
                    if (server.removeTrainTask(actualTaskInfo.getId().toString())) {
                        actualTaskInfo.setTaskState(CLOSED.getId());
                        actualTaskInfo.setTaskStateControl(0);
                    }
                    return -1;
                case RESTARTING:
                    if (server.removeTrainTask(actualTaskInfo.getId().toString())) {
                        actualTaskInfo.setTaskState(RUNNING.getId());
                        actualTaskInfo.setTaskStateControl(0);
                        actualTaskInfo.setTaskStepCode(0);
                        return 0;
                    }
                    return -1;
                default:
                    break;
            }

        switch (TaskStateEnum.getById(actualTaskInfo.getTaskState())) {
            case AUTO_RETRY:
                TrainParam param = TrainParam.trans(actualTaskInfo.getIntermediateData());
                if (Objects.isNull(param)) {
                    log.error("Unknown error happened on 'AUTO_RETRY' state,the train param is incorrect or step controller run with wrong order");
                    actualTaskInfo.setTaskState(INTERRUPT.getId());
                    return -1;
                }
                if (server.accessApply(actualTaskInfo.getId().toString(), null)
                        && server.setProcessParam(actualTaskInfo.getId().toString(), param.values())) {
                    actualTaskInfo.setTaskState(RUNNING.getId());
                    break;
                } else return -1;
            case INIT:
                actualTaskInfo.setTaskState(RUNNING.getId());
                break;
            case PAUSED:
            case CLOSED:
                return -1;
            case FINISHED:
            case INTERRUPT:
                server.removeTrainTask(actualTaskInfo.getId().toString());
                return -1;
            default:
                break;
        }
        //是否可执行下一步
        int nextStep = actualTaskInfo.getTaskStepCode();
        int control = server.executeStepControl(actualTaskInfo.getId().toString(), actualTaskInfo.getTaskStepCode() + 1);
        switch (TrainStepState.valueOf(control)) {
            case DEFAULT:
            case WAIT:
                return -1;
            case ALLOW:
                nextStep++;//由服务端控制是否有下一步操作
                break;
            case INTERRUPT:
                actualTaskInfo.setTaskState(INTERRUPT.getId());
                return -1;
            case AGO:
                break;
            case RETRY:
                actualTaskInfo.setTaskState(AUTO_RETRY.getId());
                return -1;
            case FINISH:
                actualTaskInfo.setTaskState(FINISHED.getId());
                return -1;
            default:
                log.error("Train step controller receive a unexpected 'control code'-->{}", control);
                return -1;
        }
        //单步控制逻辑
        if (actualTaskInfo.getEnableStepWait() == 1 && nextStep > actualTaskInfo.getTaskStepCode()) {
            actualTaskInfo.setTaskState(PAUSED.getId());
            return -1;

        } else {
            actualTaskInfo.setTaskStepCode(nextStep);
            return nextStep;
        }
    }

}
